.timetable-plan .row, .timetable-plan .col {
    display: flex;
    padding: 0rem;
}

.timetable-plan .row {
    margin-bottom: 0rem;
}

.lesson-card, .timetable-title-card {
    margin: 0;
    display: flex;
    flex-grow: 1;
    min-height: 40px;
    box-shadow: none;
    border: 1px solid black;
    margin-right: -1px;
    margin-top: -1px;
    border-radius: 0px;
    font-size: 11px;
}
.lesson-card .card-content > div {
    padding: 1px;
}

.card .card-title {
    font-size: 18px;
}

.timetable-title-card .card-content {
    padding: 7px;
}
