{# -*- engine:django -*- #}

{% load i18n static humanize %}

<div class="card">
  <div class="card-content">
    <span class="card-title">
      {% blocktrans with day=day|naturalday:"l" %}
        My timetable for {{ day }}
      {% endblocktrans %}
    </span>
    <div class="timetable-plan">
      {% if has_plan %}
        {% include "chronos/partials/lessons_col.html" with lesson_periods=lesson_periods %}
      {% else %}
        <figure class="alert warning">
          <i class="material-icons iconify left" data-icon="mdi:alert-outline"></i>
          {% blocktrans %}
          There is no timetable linked to your person.
          {% endblocktrans %}
        </figure>
      {% endif %}
    </div>
  </div>
  {% if has_plan %}
    <div class="card-action">
      <a href="{% url "my_timetable" %}">{% trans "Go to smart plan" %}</a>
    </div>
  {% endif %}
</div>
