# Generated by Django 3.0.7 on 2020-06-30 10:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("chronos", "0002_school_term_validity"),
    ]

    operations = [
        migrations.RemoveField(model_name="break", name="school_term",),
        migrations.AddField(
            model_name="break",
            name="validity",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="chronos.ValidityRange",
                verbose_name="Linked validity range",
            ),
        ),
        migrations.AlterUniqueTogether(
            name="timeperiod", unique_together={("weekday", "period", "validity")},
        ),
    ]
