{% if subject.colour_fg %}
  color: {{ subject.colour_fg }};
{% endif %}
{% if subject.colour_bg and subject.colour_bg != subject.colour_fg %}
  background-color: {{ subject.colour_bg }};
{% endif %}
