# Generated by Django 3.2.13 on 2022-11-09 23:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('chronos', '0011_exam'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='chronosglobalpermissions',
            options={'managed': False, 'permissions': (('view_all_room_timetables', 'Can view all room timetables'), ('view_all_group_timetables', 'Can view all group timetables'), ('view_all_person_timetables', 'Can view all person timetables'), ('view_timetable_overview', 'Can view timetable overview'), ('view_lessons_day', 'Can view all lessons per day'), ('view_supervisions_day', 'Can view all supervisions per day'))},
        ),
    ]
