# Generated by Django 3.1.3 on 2020-12-22 12:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('chronos', '0004_substitution_extra_lesson_year'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='room',
            options={'ordering': ['name', 'short_name'],
                     'permissions': (('view_room_timetable', 'Can view room timetable'),),
                     'verbose_name': 'Room', 'verbose_name_plural': 'Rooms'},
        ),
        migrations.AlterModelOptions(
            name='chronosglobalpermissions',
            options={'managed': False, 'permissions': (('view_all_room_timetables', 'Can view all room timetables'), ('view_all_group_timetables', 'Can view all group timetables'), ('view_all_person_timetables', 'Can view all person timetables'), ('view_timetable_overview', 'Can view timetable overview'), ('view_lessons_day', 'Can view all lessons per day'))},
        ),
    ]
