# Generated by Django 3.2.3 on 2021-05-21 18:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('chronos', '0006_indexes'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='chronosglobalpermissions',
            options={'managed': False, 'permissions': (('view_all_room_timetables', 'Can view all room timetables'), ('view_all_group_timetables', 'Can view all group timetables'), ('view_all_person_timetables', 'Can view all person timetables'), ('view_timetable_overview', 'Can view timetable overview'), ('view_lessons_day', 'Can view all lessons per day'))},
        ),
    ]
